@@ -43,11 +43,7 @@
 #include "xml-util.h"
 #include "gvalue-util.h"
 
-#ifdef G_OS_WIN32
-#include <rpc.h>
-#else
-#include <uuid/uuid.h>
-#endif
+#include "guul.h"
 
 #define SUBSCRIPTION_TIMEOUT 300 /* DLNA (7.2.22.1) enforced */
 
@@ -1088,34 +1084,32 @@ subscription_response (GUPnPService *service,
         soup_message_set_status (msg, SOUP_STATUS_OK);
 }
 
+/**
+ * gupnp_get_uuid:
+ *
+ * Generate and return a new UUID.
+ *
+ * Returns: (transfer full): A newly generated UUID in string representation.
+ */
+char *
+gupnp_get_uuid (void)
+{
+        return guul_get_uuid ();
+}
+
 /* Generates a new SID */
 static char *
 generate_sid (void)
 {
-#ifdef G_OS_WIN32
         char *ret = NULL;
-        UUID uuid;
-        RPC_STATUS stat;
-        stat = UuidCreate (&uuid);
-        if (stat == RPC_S_OK) {
-                unsigned char* uuidStr = NULL;
-                stat = UuidToString (&uuid, &uuidStr);
-                if (stat == RPC_S_OK) {
-                        ret = g_strdup_printf ("uuid:%s", uuidStr);
-                        RpcStringFree (&uuidStr);
-                }
-        }
+        char *uuid;
+
+
+        uuid = guul_get_uuid ();
+        ret = g_strdup_printf ("uuid:%s", uuid);
+        g_free (uuid);
 
         return ret;
-#else
-        uuid_t id;
-        char out[39];
-
-        uuid_generate (id);
-        uuid_unparse (id, out);
-
-        return g_strdup_printf ("uuid:%s", out);
-#endif
 }
 
 /* Subscription expired */
