@@ -28,6 +28,8 @@
  *
  * GObject wrapper for xmlDoc, so that we can use refcounting and weak
  * references.
+ *
+ * Since: 0.13.0
  */
 
 #include <string.h>
@@ -39,7 +41,7 @@ G_DEFINE_TYPE (GUPnPXMLDoc,
                G_TYPE_OBJECT);
 
 static void
-gupnp_xml_doc_init (GUPnPXMLDoc *doc)
+gupnp_xml_doc_init (G_GNUC_UNUSED GUPnPXMLDoc *doc)
 {
         /* Empty */
 }
@@ -73,6 +75,8 @@ gupnp_xml_doc_class_init (GUPnPXMLDocClass *klass)
  * Create a new #GUPnPXMLDoc for @xml_doc.
  *
  * Return value: A new #GUPnPXMLDoc, or %NULL on an error
+ *
+ * Since: 0.13.0
  **/
 GUPnPXMLDoc *
 gupnp_xml_doc_new (xmlDoc *xml_doc)
@@ -96,6 +100,8 @@ gupnp_xml_doc_new (xmlDoc *xml_doc)
  * Create a new #GUPnPXMLDoc for the XML document at @path.
  *
  * Return value: A new #GUPnPXMLDoc, or %NULL on an error
+ *
+ * Since: 0.13.0
  **/
 GUPnPXMLDoc *
 gupnp_xml_doc_new_from_path (const char *path,
